/*
 * Copyright (c) 2021 Alibaba Group Holding Limited
 * Express UDP is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 *
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <malloc.h>
#include <stdbool.h>

#include "bpf.h"

#include "log.h"

#define pr_err(b,   fmt, ...) logerr(b->log,   fmt, ##__VA_ARGS__)
#define pr_warn(b,  fmt, ...) logwrn(b->log,   fmt, ##__VA_ARGS__)
#define pr_debug(b, fmt, ...) logdebug(b->log, fmt, ##__VA_ARGS__)

void bpf_close(struct bpf *b)
{
	bpf_object__close(b->obj);
}

int bpf_map_get(struct bpf *b, const char *name)
{
	struct bpf_map *map = bpf_object__find_map_by_name(b->obj, name);
	if (!map) {
		pr_err(b, "bpf_object__find_map_by_name: %s\n", name);
		return -1;
	}

	return bpf_map__fd(map);
}
